import os
import pickle

from sigal.gallery import Gallery, Image
from sigal.plugins import extended_caching

CURRENT_DIR = os.path.dirname(__file__)


def test_save_cache(settings, tmpdir):
    settings["destination"] = str(tmpdir)
    gal = Gallery(settings, ncpu=1)
    extended_caching.save_cache(gal)

    cachePath = os.path.join(settings["destination"], ".metadata_cache")

    assert os.path.isfile(cachePath)

    with open(cachePath, "rb") as cacheFile:
        cache = pickle.load(cacheFile)

    # test exif
    album = gal.albums["exifTest"]
    cache_img = cache["exifTest/21.jpg"]
    assert cache_img["exif"] == album.medias[0].exif
    assert "markdown_metadata" not in cache_img
    assert cache_img["file_metadata"] == album.medias[0].file_metadata

    cache_img = cache["exifTest/22.jpg"]
    assert cache_img["exif"] == album.medias[1].exif
    assert "markdown_metadata" not in cache_img
    assert cache_img["file_metadata"] == album.medias[1].file_metadata

    cache_img = cache["exifTest/noexif.png"]
    assert cache_img["exif"] == album.medias[2].exif
    assert "markdown_metadata" not in cache_img
    assert cache_img["file_metadata"] == album.medias[2].file_metadata

    # test iptc and md
    album = gal.albums["iptcTest"]
    assert cache["iptcTest/_index"]["markdown_metadata"] == album.markdown_metadata

    cache_img = cache["iptcTest/1.jpg"]
    assert cache_img["file_metadata"] == album.medias[0].file_metadata
    assert "markdown_metadata" not in cache_img

    cache_img = cache["iptcTest/2.jpg"]
    assert cache_img["markdown_metadata"] == album.medias[1].markdown_metadata

    # test if file disappears
    gal.albums["exifTest"].medias.append(Image("foooo.jpg", "exifTest", settings))
    extended_caching.save_cache(gal)
    with open(cachePath, "rb") as cacheFile:
        cache = pickle.load(cacheFile)
    assert "exifTest/foooo.jpg" not in cache


def test_restore_cache(settings, tmpdir):
    settings["destination"] = str(tmpdir)
    gal1 = Gallery(settings, ncpu=1)
    gal2 = Gallery(settings, ncpu=1)
    extended_caching.save_cache(gal1)
    extended_caching._restore_cache(gal2)
    assert gal1.metadataCache == gal2.metadataCache

    # test bad cache
    cachePath = os.path.join(settings["destination"], ".metadata_cache")
    with open(cachePath, "w") as f:
        f.write("bad pickle file")

    extended_caching._restore_cache(gal2)
    assert gal2.metadataCache == {}


def test_load_exif(settings, tmpdir):
    settings["destination"] = str(tmpdir)
    gal1 = Gallery(settings, ncpu=1)
    gal1.albums["exifTest"].medias[2].exif = "blafoo"
    # set mod_date in future, to force these values
    gal1.metadataCache = {
        "exifTest/21.jpg": {"exif": "Foo", "mod_date": 100000000000},
        "exifTest/22.jpg": {"exif": "Bar", "mod_date": 100000000000},
    }

    extended_caching.load_metadata(gal1.albums["exifTest"])

    assert gal1.albums["exifTest"].medias[0].exif == "Foo"
    assert gal1.albums["exifTest"].medias[1].exif == "Bar"
    assert gal1.albums["exifTest"].medias[2].exif == "blafoo"

    # check if setting gallery.metadataCache works
    gal2 = Gallery(settings, ncpu=1)
    extended_caching.save_cache(gal1)
    extended_caching.load_metadata(gal2.albums["exifTest"])

    assert gal2.albums["exifTest"].medias[0].exif == "Foo"
    assert gal2.albums["exifTest"].medias[1].exif == "Bar"
    assert gal2.albums["exifTest"].medias[2].exif == "blafoo"


def test_load_metadata_missing(settings, tmpdir):
    settings["destination"] = str(tmpdir)
    gal = Gallery(settings, ncpu=1)
    extended_caching.save_cache(gal)
    assert gal.metadataCache

    # test if file disappears
    gal.albums["exifTest"].medias.append(Image("foooo.jpg", "exifTest", settings))

    # set mod_date to -1 to force cache update
    gal.metadataCache = {
        "exifTest/_index": {
            "mod_date": -1,
        },
        "exifTest/21.jpg": {"exif": "Foo", "mod_date": -1},
        "exifTest/foooo.jpg": {"exif": "Foo"},
        "dir1/test2/22.jpg": {
            "exif": "Bar",
            "mod_date": 100000000000,
            "meta_mod_date": -1,
            "markdown_metadata": "Bar",
        },
    }
    # errors should all be caught
    extended_caching.load_metadata(gal.albums["exifTest"])
    assert gal.albums["exifTest"].medias[0].exif != "Foo"
    assert gal.albums["exifTest"].medias[-1].exif != "Foo"

    extended_caching.load_metadata(gal.albums["dir1/test2"])
    assert gal.albums["dir1/test2"].medias[1].exif == "Bar"
    assert gal.albums["dir1/test2"].medias[1].markdown_metadata != "Bar"



import datetime
import logging
import os
import re
from os.path import join

import pytest
from PIL import Image as PILImage

from sigal.gallery import Album, Gallery, Image, Media, Video
from sigal.video import SubprocessException

try:
    from pillow_heif import HeifImagePlugin  # noqa: F401

    HAS_HEIF = True
except ImportError:
    HAS_HEIF = False

CURRENT_DIR = os.path.dirname(__file__)

REF = {
    "dir1": {
        "title": "An example gallery",
        "name": "dir1",
        "thumbnail": "dir1/test1/thumbnails/11.tn.jpg",
        "subdirs": ["test1", "test2", "test3"],
        "medias": [],
    },
    "dir1/test1": {
        "title": "An example sub-category",
        "name": "test1",
        "thumbnail": "test1/thumbnails/11.tn.jpg",
        "subdirs": [],
        "medias": [
            "11.jpg",
            "CMB_Timeline300_no_WMAP.jpg",
            "flickr_jerquiaga_2394751088_cc-by-nc.jpg",
            "example.gif",
        ],
    },
    "dir1/test2": {
        "title": "test2",
        "name": "test2",
        "thumbnail": "test2/thumbnails/21.tn.tiff",
        "subdirs": [],
        "medias": ["21.tiff", "22.jpg", "CMB_Timeline300_no_WMAP.jpg"],
    },
    "dir1/test3": {
        "title": "01 First title alphabetically",
        "name": "test3",
        "thumbnail": "test3/thumbnails/3.tn.jpg",
        "subdirs": [],
        "medias": ["3.jpg"],
    },
    "dir2": {
        "title": "Another example gallery with a very long name",
        "name": "dir2",
        "thumbnail": "dir2/thumbnails/m57_the_ring_nebula-587px.tn.jpg",
        "subdirs": [],
        "medias": [
            "KeckObservatory20071020.jpg",
            "Hubble Interacting Galaxy NGC 5257.jpg",
            "Hubble ultra deep field.jpg",
            "m57_the_ring_nebula-587px.jpg",
        ],
    },
    "accentué": {
        "title": "accentué",
        "name": "accentué",
        "thumbnail": "accentu%C3%A9/thumbnails/h%C3%A9lico%C3%AFde.tn.jpg",
        "subdirs": [],
        "medias": ["hélicoïde.jpg", "11.jpg"],
    },
    "video": {
        "title": "video",
        "name": "video",
        "thumbnail": "video/thumbnails/example%20video.tn.jpg",
        "subdirs": [],
        "medias": ["example video.ogv"],
    },
    "webp": {
        "title": "webp",
        "name": "webp",
        "thumbnail": "webp/thumbnails/_MG_7805_lossy80.tn.webp",
        "subdirs": [],
        "medias": ["_MG_7805_lossy80.webp", "_MG_7808_lossy80.webp"],
    },
}


def test_media(settings):
    m = Media("11.jpg", "dir1/test1", settings)
    path = join("dir1", "test1")
    file_path = join(path, "11.jpg")
    thumb = join("thumbnails", "11.tn.jpg")

    assert m.dst_filename == "11.jpg"
    assert m.src_path == join(settings["source"], file_path)
    assert m.dst_path == join(settings["destination"], file_path)
    assert m.thumb_name == thumb
    assert m.thumb_path == join(settings["destination"], path, thumb)
    assert m.title == "Foo Bar"
    assert m.description.startswith(
        "<p>This is a <em>funny</em> <strong>description</strong> of this image</p>"
    )

    assert repr(m) == f"<Media>('{file_path}')"
    assert str(m) == file_path


def test_media_orig(settings, tmpdir):
    settings["keep_orig"] = False
    m = Media("11.jpg", "dir1/test1", settings)
    assert m.big is None

    settings["keep_orig"] = True
    settings["destination"] = str(tmpdir)

    m = Image("11.jpg", "dir1/test1", settings)
    assert m.big == "original/11.jpg"

    m = Video("example video.ogv", "video", settings)
    assert m.dst_filename == "example video.webm"
    assert m.big_url == "original/example%20video.ogv"
    assert os.path.isfile(join(settings["destination"], m.path, m.big))

    settings["use_orig"] = True

    m = Image("21.jpg", "dir1/test2", settings)
    assert m.big == "21.jpg"


def test_media_iptc_override(settings):
    img_with_md = Image("2.jpg", "iptcTest", settings)
    assert img_with_md.title == "Markdown title beats iptc"
    # Markdown parsing adds formatting. Let's just focus on content
    assert "Markdown description beats iptc" in img_with_md.description
    img_no_md = Image("1.jpg", "iptcTest", settings)
    assert (
        img_no_md.title
        == "Haemostratulus clouds over Canberra - 2005-12-28 at 03-25-07"
    )
    assert (
        img_no_md.description == '"Haemo" because they look like haemoglobin '
        'cells and "stratulus" because I can\'t work out whether '
        "they're Stratus or Cumulus clouds.\nWe're driving down "
        "the main drag in Canberra so it's Parliament House that "
        "you can see at the end of the road."
    )


def test_media_img_format(settings):
    settings["img_format"] = "jpeg"
    m = Image("11.tiff", "dir1/test1", settings)
    path = join("dir1", "test1")
    thumb = join("thumbnails", "11.tn.jpeg")

    assert m.dst_filename == "11.jpeg"
    assert m.src_path == join(settings["source"], path, "11.tiff")
    assert m.dst_path == join(settings["destination"], path, "11.jpeg")
    assert m.thumb_name == thumb
    assert m.thumb_path == join(settings["destination"], path, thumb)
    assert m.title == "Foo Bar"
    assert m.description.startswith(
        "<p>This is a <em>funny</em> <strong>description</strong> of this image</p>"
    )

    file_path = join(path, "11.tiff")
    assert repr(m) == f"<Image>('{file_path}')"
    assert str(m) == file_path


def test_image(settings, tmpdir):
    settings["destination"] = str(tmpdir)
    settings["datetime_format"] = "%d/%m/%Y"
    m = Image("11.jpg", "dir1/test1", settings)
    assert m.date == datetime.datetime(2006, 1, 22, 10, 32, 42)
    assert m.exif["datetime"] == "22/01/2006"

    os.makedirs(join(settings["destination"], "dir1", "test1", "thumbnails"))
    assert m.thumbnail == join("thumbnails", "11.tn.jpg")
    assert os.path.isfile(m.thumb_path)


def test_video(settings, tmpdir):
    settings["destination"] = str(tmpdir)
    m = Video("example video.ogv", "video", settings)

    src_path = join("video", "example video.ogv")
    assert str(m) == src_path

    file_path = join("video", "example video.webm")
    assert m.dst_path == join(settings["destination"], file_path)

    os.makedirs(join(settings["destination"], "video", "thumbnails"))
    assert m.thumbnail == join("thumbnails", "example%20video.tn.jpg")
    assert os.path.isfile(m.thumb_path)


@pytest.mark.parametrize("path,album", REF.items())
def test_album(path, album, settings, tmpdir):
    gal = Gallery(settings, ncpu=1)
    a = Album(path, settings, album["subdirs"], album["medias"], gal)

    assert a.title == album["title"]
    assert a.name == album["name"]
    assert a.subdirs == album["subdirs"]
    assert a.thumbnail == album["thumbnail"]
    if path == "video":
        assert list(a.images) == []
        assert [m.dst_filename for m in a.medias] == [
            album["medias"][0].replace(".ogv", ".webm")
        ]
    else:
        assert list(a.videos) == []
        assert [m.dst_filename for m in a.medias] == album["medias"]
    assert len(a) == len(album["medias"])


def test_albums_sort(settings):
    gal = Gallery(settings, ncpu=1)
    album = REF["dir1"]
    subdirs = list(album["subdirs"])

    settings["albums_sort_reverse"] = False
    a = Album("dir1", settings, album["subdirs"], album["medias"], gal)
    a.sort_subdirs("")
    assert [alb.name for alb in a.albums] == subdirs

    settings["albums_sort_reverse"] = True
    a = Album("dir1", settings, album["subdirs"], album["medias"], gal)
    a.sort_subdirs("")
    assert [alb.name for alb in a.albums] == list(reversed(subdirs))

    titles = [im.title for im in a.albums]
    titles.sort()
    settings["albums_sort_reverse"] = False
    a = Album("dir1", settings, album["subdirs"], album["medias"], gal)
    a.sort_subdirs("title")
    assert [im.title for im in a.albums] == titles

    settings["albums_sort_reverse"] = True
    a = Album("dir1", settings, album["subdirs"], album["medias"], gal)
    a.sort_subdirs("title")
    assert [im.title for im in a.albums] == list(reversed(titles))

    orders = ["-10", "02", "03"]
    settings["albums_sort_reverse"] = False
    a = Album("dir1", settings, album["subdirs"], album["medias"], gal)
    a.sort_subdirs("meta.order")
    assert [d.meta["order"][0] for d in a.albums] == orders

    settings["albums_sort_reverse"] = True
    a = Album("dir1", settings, album["subdirs"], album["medias"], gal)
    a.sort_subdirs("meta.order")
    assert [d.meta["order"][0] for d in a.albums] == list(reversed(orders))

    settings["albums_sort_reverse"] = False
    a = Album("dir1", settings, album["subdirs"], album["medias"], gal)
    a.sort_subdirs(["meta.partialorder", "meta.order"])
    assert [d.name for d in a.albums] == list(["test1", "test2", "test3"])

    settings["albums_sort_reverse"] = False
    a = Album("dir1", settings, album["subdirs"], album["medias"], gal)
    a.sort_subdirs(["meta.partialorderb", "name"])
    assert [d.name for d in a.albums] == list(["test2", "test3", "test1"])

    settings["albums_sort_reverse"] = True
    a = Album("dir1", settings, album["subdirs"], album["medias"], gal)
    a.sort_subdirs(["meta.partialorderb", "name"])
    assert [d.name for d in a.albums] == list(["test1", "test3", "test2"])


def test_medias_sort(settings):
    gal = Gallery(settings, ncpu=1)
    album = REF["dir1/test2"]

    settings["medias_sort_reverse"] = True
    a = Album("dir1/test2", settings, album["subdirs"], album["medias"], gal)
    a.sort_medias(settings["medias_sort_attr"])
    assert [im.dst_filename for im in a.images] == list(reversed(album["medias"]))

    settings["medias_sort_attr"] = "date"
    settings["medias_sort_reverse"] = False
    a = Album("dir1/test2", settings, album["subdirs"], album["medias"], gal)
    a.sort_medias(settings["medias_sort_attr"])
    assert a.medias[0].src_filename == "22.jpg"

    settings["medias_sort_attr"] = "meta.order"
    settings["medias_sort_reverse"] = False
    a = Album("dir1/test2", settings, album["subdirs"], album["medias"], gal)
    a.sort_medias(settings["medias_sort_attr"])
    assert [im.dst_filename for im in a.images] == [
        "21.tiff",
        "22.jpg",
        "CMB_Timeline300_no_WMAP.jpg",
    ]


def test_gallery(settings, tmp_path, caplog):
    "Test the Gallery class."

    caplog.set_level("ERROR")
    settings["destination"] = str(tmp_path)
    settings["user_css"] = str(tmp_path / "my.css")
    settings["webm_options"] = ["-missing-option", "foobar"]
    gal = Gallery(settings, ncpu=1)

    gal.build()

    if HAS_HEIF:
        assert re.match(r"CSS file .* could not be found", caplog.records[3].message)
    else:
        assert re.match(r"CSS file .* could not be found", caplog.records[4].message)

    with open(tmp_path / "my.css", mode="w") as f:
        f.write("color: red")

    gal.build()

    mycss = os.path.join(settings["destination"], "static", "my.css")
    assert os.path.isfile(mycss)

    out_html = os.path.join(settings["destination"], "index.html")
    assert os.path.isfile(out_html)

    with open(out_html) as f:
        html = f.read()

    assert "<title>Sigal test gallery - Sigal test gallery ☺</title>" in html
    assert '<link rel="stylesheet" href="static/my.css">' in html

    logger = logging.getLogger("sigal")
    logger.setLevel(logging.DEBUG)
    try:
        gal = Gallery(settings, ncpu=1)
        with pytest.raises(SubprocessException):
            gal.build()
    finally:
        logger.setLevel(logging.INFO)


def test_custom_theme(settings, tmp_path, caplog):
    theme_path = tmp_path / "mytheme"
    tpl_path = theme_path / "templates"

    settings["destination"] = str(tmp_path / "build")
    settings["source"] = os.path.join(settings["source"], "encryptTest")
    settings["theme"] = str(theme_path)
    settings["title"] = "My gallery"

    gal = Gallery(settings, ncpu=1)

    with pytest.raises(Exception, match="Impossible to find the theme"):
        gal.build()

    tpl_path.mkdir(parents=True)
    (theme_path / "static").mkdir(parents=True)

    with pytest.raises(SystemExit):
        gal.build()
        assert caplog.records[0].message.startswith(
            "The template album.html was not found in template folder"
        )

    with open(tpl_path / "album.html", mode="w") as f:
        f.write(""" {{ settings.title|myfilter }} """)
    with open(tpl_path / "album_list.html", mode="w") as f:
        f.write(""" {{ settings.title|myfilter }} """)
    with open(theme_path / "filters.py", mode="w") as f:
        f.write(
            """
def myfilter(value):
    return f'{value} is very nice'
"""
        )

    gal = Gallery(settings, ncpu=1)
    gal.build()

    out_html = os.path.join(settings["destination"], "index.html")
    assert os.path.isfile(out_html)

    with open(out_html) as f:
        html = f.read()

    assert "My gallery is very nice" in html


def test_gallery_max_img_pixels(settings, tmpdir, monkeypatch):
    "Test the Gallery class with the max_img_pixels setting."
    # monkeypatch is used here to reset the value to the PIL default.
    # This value does not matter, other than it is "large"
    # to show that settings['max_img_pixels'] works.
    monkeypatch.setattr("PIL.Image.MAX_IMAGE_PIXELS", 100_000_000)

    settings["source"] = os.path.join(settings["source"], "dir2")
    settings["destination"] = str(tmpdir)
    settings["max_img_pixels"] = 5000

    logger = logging.getLogger("sigal")
    logger.setLevel(logging.DEBUG)
    try:
        with pytest.raises(PILImage.DecompressionBombError):
            gal = Gallery(settings, ncpu=1)
            gal.build()

        settings["max_img_pixels"] = 100_000_000
        gal = Gallery(settings, ncpu=1)
        gal.build()
    finally:
        logger.setLevel(logging.INFO)


def test_empty_dirs(settings):
    gal = Gallery(settings, ncpu=1)
    assert "empty" not in gal.albums
    assert "dir1/empty" not in gal.albums


def test_ignores(settings, tmp_path):
    settings["source"] = os.path.join(settings["source"], "dir1")
    settings["destination"] = str(tmp_path)
    settings["ignore_directories"] = ["*test2"]
    settings["ignore_files"] = ["*.gif", "*CMB_*"]
    gal = Gallery(settings, ncpu=1)
    gal.build()

    assert not (tmp_path / "test2").exists()
    assert not (tmp_path / "test1" / "example.gif").exists()
    assert not (tmp_path / "test1" / "CMB_Timeline300_no_WMAP.jpg").exists()



import os
import sys
from unittest import mock

import pytest

from sigal.gallery import Gallery
from sigal.plugins import compress_assets
from sigal.utils import init_plugins

CURRENT_DIR = os.path.dirname(__file__)


def make_gallery(settings, tmpdir, method):
    settings["source"] = os.path.join(settings["source"], "dir1")
    settings["destination"] = str(tmpdir)
    # Really speed up testing
    settings["use_orig"] = True
    settings["plugins"] = ["sigal.plugins.compress_assets"]

    # Set method
    settings.setdefault("compress_assets_options", {})["method"] = method

    compress_options = compress_assets.DEFAULT_SETTINGS.copy()
    # The key was created by the previous setdefault if needed
    compress_options.update(settings["compress_assets_options"])

    init_plugins(settings)
    gal = Gallery(settings)
    gal.build()

    return compress_options


def walk_destination(destination, suffixes, compress_suffix):
    for path, dirs, files in os.walk(destination):
        for file in files:
            original_filename = os.path.join(path, file)
            compressed_filename = f"{os.path.join(path, file)}.{compress_suffix}"
            path_exists = os.path.exists(compressed_filename)
            file_ext = os.path.splitext(file)[1][1:]
            if file_ext in suffixes:
                assert path_exists
                assert (
                    os.stat(original_filename).st_mtime
                    <= os.stat(compressed_filename).st_mtime
                )
            else:
                assert not path_exists


@pytest.mark.parametrize(
    "method,compress_suffix,test_import",
    [("gzip", "gz", None), ("zopfli", "gz", "zopfli.gzip"), ("brotli", "br", "brotli")],
)
def test_compress(
    disconnect_signals, settings, tmpdir, method, compress_suffix, test_import
):
    if test_import:
        pytest.importorskip(test_import)

    # Compress twice to test compression skip based on mtime
    for _ in range(2):
        compress_options = make_gallery(settings, tmpdir, method)
        walk_destination(
            settings["destination"], compress_options["suffixes"], compress_suffix
        )


@pytest.mark.parametrize(
    "method,compress_suffix,mask",
    [
        ("zopfli", "gz", "zopfli.gzip"),
        ("brotli", "br", "brotli"),
        ("__does_not_exist__", "br", None),
    ],
)
def test_failed_compress(
    disconnect_signals, settings, tmpdir, method, compress_suffix, mask
):
    # See https://medium.com/python-pandemonium/how-to-test-your-imports-1461c1113be1
    with mock.patch.dict(sys.modules, {mask: None}):
        make_gallery(settings, tmpdir, method)
        walk_destination(
            settings["destination"],
            [],
            compress_suffix,  # No file should be compressed
        )



import os

from sigal import __version__

# -- General configuration ----------------------------------------------------

# If your documentation needs a minimal Sphinx version, state it here.
# needs_sphinx = '1.0'

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom ones.
extensions = ["sphinx.ext.autodoc", "sphinx.ext.extlinks"]

extlinks = {"issue": ("https://github.com/saimn/sigal/issues/%s", "#%s")}

# Add any paths that contain templates here, relative to this directory.
templates_path = ["_templates"]

# The suffix of source filenames.
source_suffix = ".rst"

# The encoding of source files.
# source_encoding = 'utf-8-sig'

# The master toctree document.
master_doc = "index"

# General information about the project.
project = "Sigal"
copyright = "2012-2023, Simon Conseil"

# The version info for the project you're documenting, acts as replacement for
# |version| and |release|, also used in various other places throughout the
# built documents.
release = __version__
version = __version__

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
# language = None

# There are two options for replacing |today|: either, you set today to some
# non-false value, then it is used:
# today = ''
# Else, today_fmt is used as the format for a strftime call.
# today_fmt = '%B %d, %Y'

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
exclude_patterns = ["_build"]

# If true, '()' will be appended to :func: etc. cross-reference text.
# add_function_parentheses = True

# If true, the current module name will be prepended to all description
# unit titles (such as .. function::).
# add_module_names = True

# If true, sectionauthor and moduleauthor directives will be shown in the
# output. They are ignored by default.
# show_authors = False

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = "sphinx"
pygments_dark_style = "monokai"

# A list of ignored prefixes for module index sorting.
# modindex_common_prefix = []


# -- Options for HTML output --------------------------------------------------

# Define the canonical URL if you are using a custom domain on Read the Docs
html_baseurl = os.environ.get("READTHEDOCS_CANONICAL_URL", "")

# Tell Jinja2 templates the build is running on Read the Docs
html_context = {"READTHEDOCS": os.environ.get("READTHEDOCS", "") == "True"}

html_theme = "furo"

html_theme_options = {
    "source_repository": "https://github.com/saimn/sigal/",
    "source_branch": "main",
    "source_directory": "docs/",
    "footer_icons": [
        {
            "name": "GitHub",
            "url": "https://github.com/saimn/sigal",
            "html": """
                <svg stroke="currentColor" fill="currentColor" stroke-width="0" viewBox="0 0 16 16">
                    <path fill-rule="evenodd" d="M8 0C3.58 0 0 3.58 0 8c0 3.54 2.29 6.53 5.47 7.59.4.07.55-.17.55-.38 0-.19-.01-.82-.01-1.49-2.01.37-2.53-.49-2.69-.94-.09-.23-.48-.94-.82-1.13-.28-.15-.68-.52-.01-.53.63-.01 1.08.58 1.23.82.72 1.21 1.87.87 2.33.66.07-.52.28-.87.51-1.07-1.78-.2-3.64-.89-3.64-3.95 0-.87.31-1.59.82-2.15-.08-.2-.36-1.02.08-2.12 0 0 .67-.21 2.2.82.64-.18 1.32-.27 2-.27.68 0 1.36.09 2 .27 1.53-1.04 2.2-.82 2.2-.82.44 1.1.16 1.92.08 2.12.51.56.82 1.27.82 2.15 0 3.07-1.87 3.75-3.65 3.95.29.25.54.73.54 1.48 0 1.07-.01 1.93-.01 2.2 0 .21.15.46.55.38A8.013 8.013 0 0 0 16 8c0-4.42-3.58-8-8-8z"></path>
                </svg>
            """,  # noqa: E501
            "class": "",
        },
    ],
}

# Output file base name for HTML help builder.
htmlhelp_basename = "Sigaldoc"



import os
from unittest.mock import patch

import pytest
from PIL import Image as PILImage

from sigal.gallery import Image
from sigal.image import (
    generate_image,
    generate_thumbnail,
    get_exif_data,
    get_exif_tags,
    get_image_metadata,
    get_iptc_data,
    get_size,
    process_image,
)
from sigal.log import init_logging
from sigal.settings import Status, create_settings

CURRENT_DIR = os.path.dirname(__file__)
SRCDIR = os.path.join(CURRENT_DIR, "sample", "pictures")
TEST_IMAGE = "KeckObservatory20071020.jpg"
SRCFILE = os.path.join(SRCDIR, "dir2", TEST_IMAGE)

TEST_GIF_IMAGE = "example.gif"
SRC_GIF_FILE = os.path.join(SRCDIR, "dir1", "test1", TEST_GIF_IMAGE)


def test_process_image(tmpdir):
    "Test the process_image function."

    status = process_image(Image("foo.txt", "bar", create_settings()))
    assert status == Status.FAILURE

    settings = create_settings(
        img_processor="ResizeToFill",
        make_thumbs=False,
        source=os.path.join(SRCDIR, "dir2"),
        destination=str(tmpdir),
    )
    image = Image(TEST_IMAGE, ".", settings)
    status = process_image(image)
    assert status == Status.SUCCESS
    with PILImage.open(os.path.join(str(tmpdir), TEST_IMAGE)) as im:
        assert im.size == settings["img_size"]


def test_generate_image(tmpdir):
    "Test the generate_image function."

    dstfile = str(tmpdir.join(TEST_IMAGE))
    for i, size in enumerate([(600, 600), (300, 200)]):
        settings = create_settings(
            img_size=size, img_processor="ResizeToFill", copy_exif_data=True
        )
        options = None if i == 0 else {"quality": 85}
        generate_image(SRCFILE, dstfile, settings, options=options)
        with PILImage.open(dstfile) as im:
            assert im.size == size


def test_generate_image_imgformat(tmpdir):
    "Test the effects of the img_format setting on generate_image."

    dstfile = str(tmpdir.join(TEST_IMAGE))
    for i, outfmt in enumerate(["JPEG", "PNG", "TIFF"]):
        settings = create_settings(
            img_size=(300, 300),
            img_processor="ResizeToFill",
            copy_exif_data=True,
            img_format=outfmt,
        )
        options = {"quality": 85}
        generate_image(SRCFILE, dstfile, settings, options=options)
        with PILImage.open(dstfile) as im:
            assert im.format == outfmt


def test_resize_image_portrait(tmpdir):
    """Test that the area is the same regardless of aspect ratio."""
    size = (300, 200)
    settings = create_settings(img_size=size)

    portrait_image = "m57_the_ring_nebula-587px.jpg"
    portrait_src = os.path.join(
        CURRENT_DIR, "sample", "pictures", "dir2", portrait_image
    )
    portrait_dst = str(tmpdir.join(portrait_image))

    generate_image(portrait_src, portrait_dst, settings)
    with PILImage.open(portrait_dst) as im:
        # In the default mode, PILKit resizes in a way to never make an image
        # smaller than either of the lengths, the other is scaled accordingly.
        # Hence we test that the shorter side has the smallest length.
        assert im.size[0] == 200

    landscape_image = "KeckObservatory20071020.jpg"
    landscape_src = os.path.join(
        CURRENT_DIR, "sample", "pictures", "dir2", landscape_image
    )
    landscape_dst = str(tmpdir.join(landscape_image))

    generate_image(landscape_src, landscape_dst, settings)
    with PILImage.open(landscape_dst) as im:
        assert im.size[1] == 200


@pytest.mark.parametrize(
    ("image", "path"), [(TEST_IMAGE, SRCFILE), (TEST_GIF_IMAGE, SRC_GIF_FILE)]
)
def test_generate_image_passthrough(tmpdir, image, path):
    "Test the generate_image function with use_orig=True."

    dstfile = str(tmpdir.join(image))
    settings = create_settings(use_orig=True)
    generate_image(path, dstfile, settings)
    # Check the file was copied, not (sym)linked
    st_src = os.stat(path)
    st_dst = os.stat(dstfile)
    assert st_src.st_size == st_dst.st_size
    assert not os.path.samestat(st_src, st_dst)


def test_generate_image_passthrough_symlink(tmpdir):
    "Test the generate_image function with use_orig=True and orig_link=True."

    dstfile = str(tmpdir.join(TEST_IMAGE))
    settings = create_settings(use_orig=True, orig_link=True)
    generate_image(SRCFILE, dstfile, settings)
    # Check the file was symlinked
    assert os.path.islink(dstfile)
    assert os.path.samefile(SRCFILE, dstfile)


def test_generate_image_processor(tmpdir):
    "Test generate_image with a wrong processor name."

    init_logging("sigal")
    dstfile = str(tmpdir.join(TEST_IMAGE))
    settings = create_settings(img_size=(200, 200), img_processor="WrongMethod")

    with pytest.raises(SystemExit):
        generate_image(SRCFILE, dstfile, settings)


@pytest.mark.parametrize(
    ("image", "path", "wide_size", "high_size"),
    [
        (TEST_IMAGE, SRCFILE, (200, 133), (150, 100)),
        (TEST_GIF_IMAGE, SRC_GIF_FILE, (134, 150), (150, 168)),
    ],
)
def test_generate_thumbnail(tmpdir, image, path, wide_size, high_size):
    "Test the generate_thumbnail function."

    dstfile = str(tmpdir.join(image))
    for size in [(200, 150), (150, 200)]:
        generate_thumbnail(path, dstfile, size)
        with PILImage.open(dstfile) as im:
            assert im.size == size

    for size, thumb_size in [((200, 150), wide_size), ((150, 200), high_size)]:
        generate_thumbnail(path, dstfile, size, fit=False)
        with PILImage.open(dstfile) as im:
            assert im.size == thumb_size


def test_get_exif_tags():
    test_image = "11.jpg"
    src_file = os.path.join(
        CURRENT_DIR, "sample", "pictures", "dir1", "test1", test_image
    )
    data = get_exif_data(src_file)
    simple = get_exif_tags(data, datetime_format="%d/%m/%Y")
    assert simple["fstop"] == 3.9
    assert simple["focal"] == 12.0
    assert simple["iso"] == 50
    assert simple["Make"] == "NIKON"
    assert simple["datetime"] == "22/01/2006"
    assert simple["exposure"] == "100603/100000000"

    data = {"FNumber": [1, 0], "FocalLength": [1, 0], "ExposureTime": 10}
    simple = get_exif_tags(data)
    assert "fstop" not in simple
    assert "focal" not in simple
    assert simple["exposure"] == "10"

    data = {
        "ExposureTime": "--",
        "DateTimeOriginal": "---",
        "GPSInfo": {
            "GPSLatitude": ((34, 0), (1, 0), (4500, 100)),
            "GPSLatitudeRef": "N",
            "GPSLongitude": ((116, 0), (8, 0), (3900, 100)),
            "GPSLongitudeRef": "W",
        },
    }
    simple = get_exif_tags(data)
    assert "exposure" not in simple
    assert "datetime" not in simple
    assert "gps" not in simple


def test_get_heic_exif_tags():
    pytest.importorskip("pillow_heif")
    test_image = "outdoor.heic"
    src_file = os.path.join(
        CURRENT_DIR, "sample", "pictures", "dir1", "test1", test_image
    )
    data = get_exif_data(src_file)
    simple = get_exif_tags(data, datetime_format="%d/%m/%Y")
    assert simple["Make"] == "samsung"
    assert simple["datetime"] == "01/09/2024"


def test_get_iptc_data(caplog):
    test_image = "1.jpg"
    src_file = os.path.join(CURRENT_DIR, "sample", "pictures", "iptcTest", test_image)
    data = get_iptc_data(src_file)
    # Title
    assert (
        data["title"]
        == "Haemostratulus clouds over Canberra - " + "2005-12-28 at 03-25-07"
    )
    # Description
    assert (
        data["description"]
        == '"Haemo" because they look like haemoglobin '
        + 'cells and "stratulus" because I can\'t work out whether '
        + "they're Stratus or Cumulus clouds.\nWe're driving down "
        + "the main drag in Canberra so it's Parliament House that "
        + "you can see at the end of the road."
    )
    # This file has no IPTC data
    test_image = "21.jpg"
    src_file = os.path.join(CURRENT_DIR, "sample", "pictures", "exifTest", test_image)
    assert get_iptc_data(src_file) == {}

    # Headline
    test_image = "3.jpg"
    src_file = os.path.join(CURRENT_DIR, "sample", "pictures", "iptcTest", test_image)
    data = get_iptc_data(src_file)
    assert data["headline"] == "Ring Nebula, M57"

    # Test catching the SyntaxError -- assert output
    with patch("sigal.image.IptcImagePlugin.getiptcinfo", side_effect=SyntaxError):
        get_iptc_data(src_file)
        assert ["IPTC Error in"] == [log.message[:13] for log in caplog.records]


def test_get_image_metadata_exceptions():
    # image does not exist
    test_image = "bad_image.jpg"
    src_file = os.path.join(CURRENT_DIR, "sample", test_image)
    data = get_image_metadata(src_file)
    assert data == {"exif": {}, "iptc": {}, "size": {}}


def test_iso_speed_ratings():
    data = {"ISOSpeedRatings": ()}
    simple = get_exif_tags(data)
    assert "iso" not in simple

    data = {"ISOSpeedRatings": None}
    simple = get_exif_tags(data)
    assert "iso" not in simple

    data = {"ISOSpeedRatings": 125}
    simple = get_exif_tags(data)
    assert "iso" in simple


def test_exif_copy(tmpdir):
    "Test if EXIF data can transferred copied to the resized image."

    test_image = "11.jpg"
    src_file = os.path.join(
        CURRENT_DIR, "sample", "pictures", "dir1", "test1", test_image
    )
    dst_file = str(tmpdir.join(test_image))

    settings = create_settings(img_size=(300, 400), copy_exif_data=True)
    generate_image(src_file, dst_file, settings)
    simple = get_exif_tags(get_exif_data(dst_file))
    assert simple["iso"] == 50

    settings["copy_exif_data"] = False
    generate_image(src_file, dst_file, settings)
    simple = get_exif_tags(get_exif_data(dst_file))
    assert not simple


def test_exif_gps(tmpdir):
    """Test reading out correct geo tags"""

    test_image = "flickr_jerquiaga_2394751088_cc-by-nc.jpg"
    src_file = os.path.join(
        CURRENT_DIR, "sample", "pictures", "dir1", "test1", test_image
    )
    dst_file = str(tmpdir.join(test_image))

    settings = create_settings(img_size=(400, 300), copy_exif_data=True)
    generate_image(src_file, dst_file, settings)
    simple = get_exif_tags(get_exif_data(dst_file))
    assert "gps" in simple

    lat = 34.029167
    lon = -116.144167

    assert abs(simple["gps"]["lat"] - lat) < 0.0001
    assert abs(simple["gps"]["lon"] - lon) < 0.0001


def test_get_size(tmpdir):
    """Test reading out image size"""

    test_image = "flickr_jerquiaga_2394751088_cc-by-nc.jpg"
    src_file = os.path.join(
        CURRENT_DIR, "sample", "pictures", "dir1", "test1", test_image
    )

    result = get_size(src_file)
    assert result == {"height": 800, "width": 600}


def test_get_size_with_invalid_path(tmpdir):
    """Test reading out image size with a missing file"""

    test_image = "missing-file.jpg"
    src_file = os.path.join(CURRENT_DIR, test_image)

    result = get_size(src_file)
    assert result is None



import os
import shutil

import blinker
import PIL
import pytest

from sigal import signals
from sigal.settings import read_settings

CURRENT_DIR = os.path.abspath(os.path.dirname(__file__))
BUILD_DIR = os.path.join(CURRENT_DIR, "sample", "_build")


@pytest.fixture(scope="session", autouse=True)
def remove_build():
    """Ensure that build directory does not exists before each test."""
    if os.path.exists(BUILD_DIR):
        shutil.rmtree(BUILD_DIR)


@pytest.fixture
def settings():
    """Read the sample config file."""
    return read_settings(os.path.join(CURRENT_DIR, "sample", "sigal.conf.py"))


@pytest.fixture()
def disconnect_signals():
    # Reset plugins
    yield None
    for name in dir(signals):
        if not name.startswith("_"):
            try:
                sig = getattr(signals, name)
                if isinstance(sig, blinker.Signal):
                    sig.receivers.clear()
            except Exception:
                pass


def pytest_report_header(config):
    return f"project deps: Pillow-{PIL.__version__}"



import os

from sigal.gallery import Gallery
from sigal.utils import init_plugins

CURRENT_DIR = os.path.dirname(__file__)


def _build_with_plugin(
    settings, input_path, output_path, plugin, **additional_settings
):
    settings["source"] = os.path.join(settings["source"], input_path)
    settings["destination"] = str(output_path)
    settings["plugins"] = [plugin]
    settings.update(additional_settings)

    init_plugins(settings)
    gal = Gallery(settings, ncpu=1)
    gal.build()
    return gal


def test_media_page(settings, tmp_path, disconnect_signals):
    _build_with_plugin(
        settings, "dir2", tmp_path, "sigal.plugins.media_page", theme="colorbox"
    )
    assert (tmp_path / "KeckObservatory20071020.jpg.html").is_file()


def test_nomedia(settings, tmp_path, disconnect_signals):
    _build_with_plugin(settings, "nomedia", tmp_path, "sigal.plugins.nomedia")

    for path, dirs, files in os.walk(str(tmp_path)):
        assert "ignore" not in path
        for file in files:
            assert "ignore" not in file


def test_nonmedia_files(settings, tmp_path, disconnect_signals):
    _build_with_plugin(
        settings,
        "nonmedia_files",
        tmp_path,
        "sigal.plugins.nonmedia_files",
        nonmedia_files_options={"thumb_bg_color": "red"},
    )
    assert (tmp_path / "dummy.pdf").is_file()
    assert (tmp_path / "thumbnails" / "dummy.tn.jpg").is_file()


def test_titleregexp(settings, tmp_path, disconnect_signals):
    conf = {
        "regexp": [
            {
                "search": r"test ?(.*)",
                "replace": r"titleregexp \1",
                "substitute": [["2", "02"]],
                "break": 1,
            }
        ]
    }
    gal = _build_with_plugin(
        settings, "dir1", tmp_path, "sigal.plugins.titleregexp", titleregexp=conf
    )
    assert gal.albums["test2"].title == "titleregexp 02"



import os
from datetime import datetime
from unittest.mock import patch

import pytest

from sigal.gallery import Video
from sigal.settings import Status, create_settings
from sigal.video import generate_thumbnail, generate_video, process_video, video_size

CURRENT_DIR = os.path.dirname(__file__)
SRCDIR = os.path.join(CURRENT_DIR, "sample", "pictures")
TEST_VIDEO = "example video.ogv"
SRCFILE = os.path.join(SRCDIR, "video", TEST_VIDEO)


def test_video_size():
    size_src = video_size(SRCFILE)
    assert size_src == (240, 98)
    size_src = video_size("missing/file.mp4")
    assert size_src == (0, 0)


def test_generate_thumbnail(tmpdir):
    outname = str(tmpdir.join("test.jpg"))
    generate_thumbnail(SRCFILE, outname, (50, 50), 5)
    assert os.path.isfile(outname)


def test_process_video(tmpdir):
    base, ext = os.path.splitext(TEST_VIDEO)

    settings = create_settings(
        video_format="ogv",
        use_orig=True,
        orig_link=True,
        source=os.path.join(SRCDIR, "video"),
        destination=str(tmpdir),
    )
    video = Video(TEST_VIDEO, ".", settings)
    process_video(video)
    dstfile = str(tmpdir.join(base + ".ogv"))
    assert os.path.realpath(dstfile) == SRCFILE

    settings = create_settings(video_format="mjpg")
    assert process_video(video) == Status.FAILURE

    settings = create_settings(thumb_video_delay=-1)
    assert process_video(video) == Status.FAILURE


def test_metadata(tmpdir):
    base, ext = os.path.splitext(TEST_VIDEO)

    settings = create_settings(
        video_format="ogv",
        use_orig=True,
        orig_link=True,
        source=os.path.join(SRCDIR, "video"),
        destination=str(tmpdir),
    )
    video = Video(TEST_VIDEO, ".", settings)
    assert video.meta == {"date": ["2020-01-01T09:00:00"]}
    assert video.date == datetime(2020, 1, 1, 9, 0)


@pytest.mark.parametrize("fmt", ["webm", "mp4"])
def test_generate_video_fit(tmpdir, fmt):
    """largest fitting dimension is width"""

    base, ext = os.path.splitext(TEST_VIDEO)
    dstfile = str(tmpdir.join(base + "." + fmt))
    settings = create_settings(video_size=(125, 50), video_format=fmt)
    generate_video(SRCFILE, dstfile, settings)

    size_src = video_size(SRCFILE)
    size_dst = video_size(dstfile)

    assert size_dst[1] == 50
    # less than 2% error on ratio
    assert abs(size_dst[0] / size_dst[1] - size_src[0] / size_src[1]) < 2e-2


@pytest.mark.parametrize("fmt", ["webm", "mp4", "ogv"])
def test_generate_video_dont_enlarge(tmpdir, fmt):
    """Video dimensions should not be enlarged."""

    base, ext = os.path.splitext(TEST_VIDEO)
    dstfile = str(tmpdir.join(base + "." + fmt))
    settings = create_settings(video_size=(1000, 1000), video_format=fmt)
    generate_video(SRCFILE, dstfile, settings)
    size_src = video_size(SRCFILE)
    size_dst = video_size(dstfile)

    assert size_src == size_dst


@patch("sigal.video.generate_video_pass")
@pytest.mark.parametrize("fmt", ["webm", "mp4"])
def test_second_pass_video(mock_generate_video_pass, fmt, tmpdir):
    """Video should be run through ffmpeg."""
    base, ext = os.path.splitext(TEST_VIDEO)
    dstfile = str(tmpdir.join(base + "." + fmt))
    settings_1 = "-c:v libvpx-vp9 -b:v 0 -crf 30 -pass 1 -an -f null dev/null"
    settings_2 = f"-c:v libvpx-vp9 -b:v 0 -crf 30 -pass 2 -f {fmt}"
    settings_opts = {
        "video_size": (100, 50),
        "video_format": fmt,
        fmt + "_options": settings_1.split(" "),
        fmt + "_options_second_pass": settings_2.split(" "),
    }

    settings = create_settings(**settings_opts)
    generate_video(SRCFILE, dstfile, settings)
    call_args_list = mock_generate_video_pass.call_args_list
    # The method is called twice
    assert len(call_args_list) == 2
    # The first call to the method should have 3 args, without the outname
    args, kwargs = call_args_list[0]
    assert len(args) == 3
    # The second call to the method should have 4 args, with the outname
    args, kwargs = call_args_list[1]
    assert len(args) == 4



import os
import pickle
from io import BytesIO

import pytest

from sigal.gallery import Gallery
from sigal.plugins.encrypt import endec
from sigal.plugins.encrypt.encrypt import cache_key
from sigal.utils import init_plugins

CURRENT_DIR = os.path.dirname(__file__)


def get_key_tag(settings):
    options = settings["encrypt_options"]
    key = endec.kdf_gen_key(
        options["password"], options["kdf_salt"], options["kdf_iters"]
    )
    tag = options["gcm_tag"].encode("utf-8")
    return (key, tag)


def test_encrypt(settings, tmpdir, disconnect_signals, caplog):
    settings["source"] = os.path.join(settings["source"], "encryptTest")
    settings["destination"] = str(tmpdir)
    settings["plugins"] = ["sigal.plugins.encrypt"]

    init_plugins(settings)
    gal = Gallery(settings, ncpu=1)

    with pytest.raises(ValueError, match="no encrypt_options in settings"):
        gal.build()

    settings["encrypt_options"] = {}

    gal = Gallery(settings, ncpu=1)

    with pytest.raises(ValueError, match="no password provided"):
        gal.build()

    settings["encrypt_options"] = {
        "password": "password",
        "ask_password": True,
        "encrypt_symlinked_originals": False,
    }

    gal = Gallery(settings, ncpu=1)
    gal.build()

    # check the encrypt cache exists
    cachePath = os.path.join(settings["destination"], ".encryptCache")
    assert os.path.isfile(cachePath)

    encryptCache = None
    with open(cachePath, "rb") as cacheFile:
        encryptCache = pickle.load(cacheFile)
    assert isinstance(encryptCache, dict)

    testAlbum = gal.albums["."]
    key, tag = get_key_tag(settings)

    for media in testAlbum:
        # check if sizes are stored in cache
        assert cache_key(media) in encryptCache
        assert "size" in encryptCache[cache_key(media)]
        assert "thumb_size" in encryptCache[cache_key(media)]
        assert "encrypted" in encryptCache[cache_key(media)]

        encryptedImages = [media.dst_path, media.thumb_path]
        if settings["keep_orig"]:
            encryptedImages.append(
                os.path.join(settings["destination"], media.path, media.big)
            )

        # check if images are encrypted by trying to decrypt
        for image in encryptedImages:
            with open(image, "rb") as infile:
                with BytesIO() as outfile:
                    endec.decrypt(key, infile, outfile, tag)

    # check static files have been copied
    static = os.path.join(settings["destination"], "static")
    assert os.path.isfile(os.path.join(static, "decrypt.js"))
    assert os.path.isfile(os.path.join(static, "keycheck.txt"))
    assert os.path.isfile(os.path.join(settings["destination"], "sw.js"))

    # check keycheck file
    with open(
        os.path.join(settings["destination"], "static", "keycheck.txt"), "rb"
    ) as infile:
        with BytesIO() as outfile:
            endec.decrypt(key, infile, outfile, tag)

    caplog.clear()
    caplog.set_level("DEBUG")
    gal = Gallery(settings, ncpu=1)
    gal.build()
    # Doesn't work on Actions ...
    # assert 'Loaded cache with 34 entries' in caplog.messages
    # assert 'Loaded encryption cache with 27 entries' in caplog.messages



import os
from pathlib import Path

from sigal import utils

CURRENT_DIR = os.path.dirname(__file__)
SAMPLE_DIR = os.path.join(CURRENT_DIR, "sample")


def test_copy(tmpdir):
    filename = "KeckObservatory20071020.jpg"
    src = os.path.join(SAMPLE_DIR, "pictures", "dir2", filename)
    dst = str(tmpdir.join(filename))
    utils.copy(src, dst)
    assert os.path.isfile(dst)

    filename = "m57_the_ring_nebula-587px.jpg"
    src = os.path.join(SAMPLE_DIR, "pictures", "dir2", filename)
    dst = str(tmpdir.join(filename))
    utils.copy(src, dst, symlink=True)
    assert os.path.islink(dst)
    assert os.readlink(dst) == src

    filename = "KeckObservatory20071020.jpg"
    src = os.path.join(SAMPLE_DIR, "pictures", "dir2", filename)
    utils.copy(src, dst, symlink=True)
    assert os.path.islink(dst)
    assert os.readlink(dst) == src

    filename = "KeckObservatory20071020.jpg"
    src = os.path.join(SAMPLE_DIR, "pictures", "dir2", filename)
    dst = str(tmpdir.join(filename))
    utils.copy(src, dst, symlink=True, rellink=True)
    assert os.path.islink(dst)
    assert os.path.join(os.path.dirname(CURRENT_DIR)), os.readlink(dst) == src
    # get absolute path of the current dir plus the relative dir

    src = str(tmpdir.join("foo.txt"))
    dst = str(tmpdir.join("bar.txt"))
    p = Path(src)
    p.touch()
    p.chmod(0o444)
    utils.copy(src, dst)
    utils.copy(src, dst)


def test_force(tmpdir):
    assert utils.should_reprocess_album("Gallery/New Pics", "New Pics", False) is False
    assert utils.should_reprocess_album("Gallery/New Pics", "New Pics", True) is True
    assert (
        utils.should_reprocess_album("Gallery/New Pics", "New Pics", ["Gallery/*"])
        is True
    )
    assert (
        utils.should_reprocess_album("Gallery/New Pics", "New Pics", ["Gallery/*Pics"])
        is True
    )
    assert (
        utils.should_reprocess_album("Gallery/New Pics", "New Pics", ["Pictures/*"])
        is False
    )
    assert (
        utils.should_reprocess_album("Gallery/New Pics", "New Pics", ["New Pics"])
        is True
    )
    assert (
        utils.should_reprocess_album("Gallery/New Pics", "New Pics", ["Pictures"])
        is False
    )
    assert (
        utils.should_reprocess_album(
            "Gallery/New Pics", "New Pics", ["Pictures", "Something"]
        )
        is False
    )
    assert (
        utils.should_reprocess_album(
            "Gallery/New Pics", "New Pics", ["Pictures", "Gallery", "*Pics"]
        )
        is True
    )


def test_check_or_create_dir(tmpdir):
    path = str(tmpdir.join("new_directory"))
    utils.check_or_create_dir(path)
    assert os.path.isdir(path)


def test_url_from_path():
    assert utils.url_from_path(os.sep.join(["foo", "bar"])) == "foo/bar"


def test_url_from_windows_path(monkeypatch):
    monkeypatch.setattr("os.sep", "\\")
    path = os.sep.join(["foo", "bar"])
    assert path == r"foo\bar"
    assert utils.url_from_path(path) == "foo/bar"


def test_read_markdown():
    src = os.path.join(SAMPLE_DIR, "pictures", "dir1", "test1", "11.md")
    m = utils.read_markdown(src)
    assert m["title"] == "Foo Bar"
    assert m["meta"]["location"][0] == "Bavaria"
    assert m["description"].startswith(
        "<p>This is a <em>funny</em> <strong>description</strong> of this image</p>"
    )


def test_read_markdown_empty_file(tmpdir):
    src = tmpdir.join("file.txt")
    src.write("content")
    m = utils.read_markdown(str(src))
    assert "title" not in m
    assert m["meta"] == {}
    assert m["description"] == "<p>content</p>"

    src = tmpdir.join("empty.txt")
    src.write("")
    m = utils.read_markdown(str(src))
    assert "title" not in m
    # See https://github.com/Python-Markdown/markdown/pull/672
    # Meta attributes should always be there
    assert m["meta"] == {}
    assert m["description"] == ""


def test_is_valid_html5_video():
    assert utils.is_valid_html5_video(".webm") is True
    assert utils.is_valid_html5_video(".mpeg") is False



import os

from sigal.settings import get_thumb, read_settings

CURRENT_DIR = os.path.abspath(os.path.dirname(__file__))


def test_read_settings(settings):
    """Test that the settings are correctly read."""
    assert settings["img_size"] == (640, 480)
    assert settings["thumb_size"] == (200, 150)
    assert settings["thumb_suffix"] == ".tn"
    assert settings["source"] == os.path.join(CURRENT_DIR, "sample", "pictures")


def test_get_thumb(settings):
    """Test the get_thumb function."""
    tests = [
        ("example.jpg", "thumbnails/example.tn.jpg"),
        ("test/example.jpg", "test/thumbnails/example.tn.jpg"),
        ("test/t/example.jpg", "test/t/thumbnails/example.tn.jpg"),
    ]
    for src, ref in tests:
        assert get_thumb(settings, src) == ref

    tests = [
        ("example.webm", "thumbnails/example.tn.jpg"),
        ("test/example.mp4", "test/thumbnails/example.tn.jpg"),
        ("test/t/example.avi", "test/t/thumbnails/example.tn.jpg"),
    ]
    for src, ref in tests:
        assert get_thumb(settings, src) == ref


def test_img_sizes(tmpdir):
    """Test that image size is swaped if needed."""

    conf = tmpdir.join("sigal.conf.py")
    conf.write("thumb_size = (150, 200)")

    settings = read_settings(str(conf))
    assert settings["thumb_size"] == (200, 150)


def test_theme_path(tmpdir):
    """Test that image size is swaped if needed."""

    tmpdir.join("theme").mkdir()
    tmpdir.join("theme").join("templates").mkdir()
    conf = tmpdir.join("sigal.conf.py")
    conf.write("theme = 'theme'")

    settings = read_settings(str(conf))
    assert settings["theme"] == tmpdir.join("theme")



author = "John Doe"
title = "Sigal test gallery ☺"
source = "pictures"
thumb_suffix = ".tn"
keep_orig = True
thumb_video_delay = 5
# img_format = 'jpeg'

links = [
    ("Example link", "http://example.org"),
    ("Another link", "http://example.org"),
]

albums_sort_attr = ["meta.nokey", "nosuchattribute", "name"]

ignore_directories = [".venv*"]
files_to_copy = (("../watermark.png", "watermark.png"),)

plugins = [
    "sigal.plugins.adjust",
    "sigal.plugins.copyright",
    "sigal.plugins.extended_caching",
    "sigal.plugins.feeds",
    "sigal.plugins.watermark",
    "sigal.plugins.zip_gallery",
]
copyright = "© An example copyright message"
adjust_options = {
    "color": 0.9,
    "brightness": 1.0,
    "contrast": 1.0,
    "sharpness": 0.0,
}
watermark = "watermark.png"
watermark_position = (10, 10)
watermark_opacity = 0.3

theme = "photoswipe"
thumb_size = (200, 150)

rss_feed = {"feed_url": "http://127.0.0.1:8000/feed.rss", "nb_items": 10}
atom_feed = {"feed_url": "http://127.0.0.1:8000/feed.atom", "nb_items": 10}

# theme = 'colorbox'
# theme = 'galleria'
# thumb_size = (280, 210)
# galleria_theme = 'folio'
show_map = True



import os
import zipfile

from sigal.gallery import Gallery
from sigal.settings import read_settings
from sigal.utils import init_plugins

CURRENT_DIR = os.path.dirname(__file__)
SAMPLE_DIR = os.path.join(CURRENT_DIR, "sample")
SAMPLE_SOURCE = os.path.join(SAMPLE_DIR, "pictures")


def make_gallery(source_dir="dir1", **kwargs):
    default_conf = os.path.join(SAMPLE_DIR, "sigal.conf.py")
    settings = read_settings(default_conf)
    settings["source"] = os.path.join(SAMPLE_SOURCE, source_dir)
    settings.update(kwargs)
    init_plugins(settings)
    return Gallery(settings, ncpu=1)


def test_zipped_correctly(tmpdir):
    outpath = str(tmpdir)
    gallery = make_gallery(destination=outpath, zip_gallery="archive.zip")
    gallery.build()

    zipf = os.path.join(outpath, "test2", "archive.zip")
    assert os.path.isfile(zipf)

    zip_file = zipfile.ZipFile(zipf, "r")
    expected = ("21.tiff", "22.jpg", "CMB_Timeline300_no_WMAP.jpg")

    for filename in zip_file.namelist():
        assert filename in expected

    zip_file.close()

    assert os.path.isfile(os.path.join(outpath, "test2", "archive.zip"))


def test_not_zipped(tmpdir):
    # test that the zip file is not created when the .nozip_gallery file
    # is present
    outpath = str(tmpdir)
    gallery = make_gallery(
        destination=outpath, zip_gallery="archive.zip", source_dir="dir2"
    )
    gallery.build()
    assert not os.path.isfile(os.path.join(outpath, "archive.zip"))


def test_no_archive(tmpdir):
    outpath = str(tmpdir)
    gallery = make_gallery(destination=outpath, zip_gallery=False)
    gallery.build()

    assert not os.path.isfile(os.path.join(outpath, "test1", "archive.zip"))
    assert not os.path.isfile(os.path.join(outpath, "test2", "archive.zip"))


def test_correct_filename(tmpdir, caplog):
    caplog.set_level("ERROR")
    outpath = str(tmpdir)
    gallery = make_gallery(destination=outpath, zip_gallery=True)
    gallery.build()

    assert caplog.records[0].message == "'zip_gallery' should be set to a filename"
    assert not os.path.isfile(os.path.join(outpath, "test1", "archive.zip"))
    assert not os.path.isfile(os.path.join(outpath, "test2", "archive.zip"))



# Copyright (c) 2009-2023 - Simon Conseil
# Copyright (c)      2013 - Christophe-Marie Duquesne
# Copyright (c)      2018 - Edwin Steele

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to
# deal in the Software without restriction, including without limitation the
# rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
# sell copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
# IN THE SOFTWARE.

import importlib
import logging
import os
import shutil
import stat
import sys
import types
from datetime import datetime

from jinja2 import ChoiceLoader, Environment, FileSystemLoader, PrefixLoader
from jinja2.exceptions import TemplateNotFound

from . import signals
from .utils import url_from_path

THEMES_PATH = os.path.normpath(
    os.path.join(os.path.abspath(os.path.dirname(__file__)), "themes")
)


class AbstractWriter:
    template_file = None

    def __init__(self, settings, index_title=""):
        self.settings = settings
        self.output_dir = settings["destination"]
        self.theme = settings["theme"]
        self.index_title = index_title
        self.logger = logging.getLogger(__name__)

        # search the theme in sigal/theme if the given one does not exists
        if not os.path.exists(self.theme) or not os.path.exists(
            os.path.join(self.theme, "templates")
        ):
            self.theme = os.path.join(THEMES_PATH, self.theme)
            if not os.path.exists(self.theme):
                raise Exception(f"Impossible to find the theme {self.theme}")

        self.logger.info("Theme  : %s", self.theme)
        theme_relpath = os.path.join(self.theme, "templates")
        default_loader = FileSystemLoader(
            os.path.join(THEMES_PATH, "default", "templates")
        )

        # setup jinja env
        env_options = {"trim_blocks": True, "autoescape": True, "lstrip_blocks": True}

        loaders = [
            FileSystemLoader(theme_relpath),
            default_loader,  # implicit inheritance
            PrefixLoader({"!default": default_loader}),  # explicit one
        ]
        env = Environment(loader=ChoiceLoader(loaders), **env_options)

        # handle optional filters.py
        filters_py = os.path.join(self.theme, "filters.py")
        if os.path.exists(filters_py):
            self.logger.info("Loading filters file: %s", filters_py)
            module_spec = importlib.util.spec_from_file_location("filters", filters_py)
            mod = importlib.util.module_from_spec(module_spec)
            sys.modules["filters"] = mod
            module_spec.loader.exec_module(mod)
            for name in dir(mod):
                if isinstance(getattr(mod, name), types.FunctionType):
                    env.filters[name] = getattr(mod, name)

        try:
            self.template = env.get_template(self.template_file)
        except TemplateNotFound:
            self.logger.error(
                "The template %s was not found in template folder %s.",
                self.template_file,
                theme_relpath,
            )
            sys.exit(1)

        self.theme_path = os.path.join(self.output_dir, "static")

    def copy_theme_files(self):
        """Copy the theme files to the destination"""
        self.logger.info("Copying the theme files to the output dir")

        if os.path.isdir(self.theme_path):
            shutil.rmtree(self.theme_path)

        for static_path in (
            os.path.join(THEMES_PATH, "default", "static"),
            os.path.join(self.theme, "static"),
        ):
            shutil.copytree(static_path, self.theme_path, dirs_exist_ok=True)

            # Ensure that the theme dir is writeable
            for root, _, files in os.walk(self.theme_path):
                st = os.stat(root)
                os.chmod(root, st.st_mode | stat.S_IWUSR)
                for name in files:
                    path = os.path.join(root, name)
                    st = os.stat(path)
                    os.chmod(path, st.st_mode | stat.S_IWUSR)

        if self.settings["user_css"]:
            if not os.path.exists(self.settings["user_css"]):
                self.logger.error(
                    "CSS file %s could not be found", self.settings["user_css"]
                )
            else:
                shutil.copy(self.settings["user_css"], self.theme_path)

    def generate_context(self, album):
        """Generate the context dict for the given path."""

        from . import __url__ as sigal_link

        self.logger.info("Output album : %r", album)
        ctx = {
            "album": album,
            "index_title": self.index_title,
            "settings": self.settings,
            "sigal_link": sigal_link,
            "generated_timestamp": datetime.now().strftime(
                self.settings["datetime_format"]
            ),
            "theme": {
                "name": os.path.basename(self.theme),
                "url": url_from_path(os.path.relpath(self.theme_path, album.dst_path)),
            },
        }
        if self.settings["user_css"]:
            ctx["user_css"] = os.path.basename(self.settings["user_css"])
        return ctx

    def write(self, album):
        """Generate the HTML page and save it."""
        context = self.generate_context(album)
        signals.before_render.send(context)
        page = self.template.render(**context)
        output_file = os.path.join(album.dst_path, album.output_file)

        with open(output_file, "w", encoding="utf-8") as f:
            f.write(page)


class AlbumListPageWriter(AbstractWriter):
    """Generate an html page for a directory of albums"""

    template_file = "album_list.html"


class AlbumPageWriter(AbstractWriter):
    """Generate html pages for a directory of images."""

    template_file = "album.html"



import logging
import os
from os.path import join

from click.testing import CliRunner

from sigal.__main__ import build, init, serve, set_meta

TESTGAL = join(os.path.abspath(os.path.dirname(__file__)), "sample")


def test_init(tmpdir):
    config_file = str(tmpdir.join("sigal.conf.py"))
    runner = CliRunner()
    result = runner.invoke(init, [config_file])
    assert result.exit_code == 0
    assert result.output.startswith("Sample config file created:")
    assert os.path.isfile(config_file)

    result = runner.invoke(init, [config_file])
    assert result.exit_code == 1
    assert (
        result.output == "Found an existing config file, will abort to keep it safe.\n"
    )


def test_build(tmpdir, disconnect_signals):
    runner = CliRunner()
    config_file = str(tmpdir.join("sigal.conf.py"))
    tmpdir.mkdir("pictures")
    tmpdir = str(tmpdir)
    cwd = os.getcwd()

    try:
        result = runner.invoke(init, [config_file])
        assert result.exit_code == 0
        os.symlink(join(TESTGAL, "watermark.png"), join(tmpdir, "watermark.png"))
        os.symlink(
            join(TESTGAL, "pictures", "dir2", "KeckObservatory20071020.jpg"),
            join(tmpdir, "pictures", "KeckObservatory20071020.jpg"),
        )

        result = runner.invoke(build, ["-n", 1, "--debug", "--verbose"])
        assert (
            result.output
            == "Only one option of debug, verbose and quiet should be used\n"
        )
        assert result.exit_code == 1

        result = runner.invoke(build, ["-n", 1, "--verbose"])
        assert result.output == "Settings file not found: sigal.conf.py\n"
        assert result.exit_code == 1

        os.chdir(tmpdir)

        result = runner.invoke(build, ["foo", "-n", 1, "--quiet"])
        assert result.exit_code == 1
        assert "Input directory not found" in result.output

        result = runner.invoke(build, ["pictures", "pictures/out", "-n", 1, "--debug"])
        assert result.exit_code == 1
        assert (
            "Output directory should be outside of the input directory" in result.output
        )

        with open(config_file) as f:
            text = f.read()

        text += """
theme = 'colorbox'
files_to_copy = (('../watermark.png', 'watermark.png'),)
plugins = ['sigal.plugins.adjust', 'sigal.plugins.copyright',
           'sigal.plugins.watermark', 'sigal.plugins.feeds',
           'sigal.plugins.media_page', 'sigal.plugins.nomedia',
           'sigal.plugins.extended_caching']
copyright = "An example copyright message"
copyright_text_font = "foobar"
watermark = "watermark.png"
watermark_position = "scale"
watermark_opacity = 0.3
rss_feed = {'feed_url': 'http://example.org/feed.rss', 'nb_items': 10}
atom_feed = {'feed_url': 'http://example.org/feed.atom', 'nb_items': 10}
"""

        with open(config_file, "w") as f:
            f.write(text)

        result = runner.invoke(
            build,
            ["pictures", "build", "--title", "Testing build", "-n", 1, "--debug"],
            catch_exceptions=False,
        )
        assert result.exit_code == 0
        assert os.path.isfile(
            join(tmpdir, "build", "thumbnails", "KeckObservatory20071020.jpg")
        )
        assert os.path.isfile(join(tmpdir, "build", "feed.atom"))
        assert os.path.isfile(join(tmpdir, "build", "feed.rss"))
        assert os.path.isfile(join(tmpdir, "build", "watermark.png"))
    finally:
        os.chdir(cwd)
        # Reset logger
        logger = logging.getLogger("sigal")
        logger.handlers[:] = []
        logger.setLevel(logging.INFO)


def test_serve(tmpdir):
    config_file = str(tmpdir.join("sigal.conf.py"))
    runner = CliRunner()
    result = runner.invoke(init, [config_file])
    assert result.exit_code == 0

    result = runner.invoke(serve)
    assert result.exit_code == 2
    assert result.output.startswith("The _build directory doesn't exist")

    result = runner.invoke(serve, ["-c", config_file])
    assert result.exit_code == 1
    assert result.output.endswith("maybe try building first?\n")


def test_set_meta(tmpdir):
    testdir = tmpdir.mkdir("test")

    testfile = tmpdir.join("test.jpg")
    testfile.write("")

    runner = CliRunner()
    result = runner.invoke(set_meta, [str(testdir), "title", "testing"])

    assert result.exit_code == 0
    assert result.output.startswith("1 metadata key(s) written to")
    assert os.path.isfile(str(testdir.join("index.md")))
    assert testdir.join("index.md").read() == "Title: testing\n"

    # Run again, should give file exists error
    result = runner.invoke(set_meta, [str(testdir), "title", "testing"])
    assert result.exit_code == 2

    result = runner.invoke(
        set_meta, [str(testdir.join("non-existant.jpg")), "title", "testing"]
    )
    assert result.exit_code == 1

    result = runner.invoke(set_meta, [str(testfile), "title", "testing"])

    assert result.exit_code == 0
    assert result.output.startswith("1 metadata key(s) written to")
    assert os.path.isfile(str(tmpdir.join("test.md")))
    assert tmpdir.join("test.md").read() == "Title: testing\n"

    result = runner.invoke(set_meta, [str(testfile), "title"])

    assert result.exit_code == 1
    assert result.output.startswith("Need an even number of arguments")
